@@ -811,7 +811,7 @@ void bboxcalc(short testval, short *lowerval, short *upperval)
 void calcextents(genericptr *bboxgen, short *llx, short *lly, 
 	short *urx, short *ury)
 {
-   switch ((*bboxgen)->type) {
+   switch (ELEMENTTYPE(*bboxgen)) {
       case(POLYGON): {
          pointlist bboxpts;
          for (bboxpts = TOPOLY(bboxgen)->points; bboxpts < TOPOLY(bboxgen)->points
@@ -919,9 +919,9 @@ void calcbboxsingle(genericptr *bboxgen, objinstptr thisinst,
    /* For each screen element, compute the extents and revise bounding	*/
    /* box points, if necessary. 					*/
 
-   switch((*bboxgen)->type) {
+   switch(ELEMENTTYPE(*bboxgen)) {
 
-      case(OBJECT):
+      case(OBJINST):
 	 objinstbbox(TOOBJINST(bboxgen), npoints);
 
          for (j = 0; j < 4; j++) {
@@ -986,7 +986,7 @@ short find_object(objectptr pageobj, objectptr thisobject)
 
    for (i = 0; i < pageobj->parts; i++) {
       pelem = pageobj->plist + i;
-      if ((*pelem)->type == OBJECT) {
+      if (IS_OBJINST(*pelem)) {
 	 if ((TOOBJINST(pelem))->thisobject == thisobject)
 	    return i;
 	 else if ((j = find_object((TOOBJINST(pelem))->thisobject, thisobject)) >= 0)
@@ -1075,7 +1075,7 @@ void calcbboxinst(objinstptr thisinst)
       /* pins which do not appear outside of the object	*/
       /* contribute to the objects "schembbox".		*/
 
-      if ((*gelem)->type == LABEL) {
+      if (IS_LABEL(*gelem)) {
 	 labelptr btext = TOLABEL(gelem);
 	 if (btext->pin && !(btext->justify & PINVISIBLE)) {
 	    hasschembbox = TRUE;
@@ -1220,7 +1220,7 @@ void calcbboxvalues(objinstptr thisinst, genericptr *newelement)
 	 /* pins which do not appear outside of the object 	*/
 	 /* are ignored now---will be computed per instance.	*/
 
-	 if ((*bboxgen)->type == LABEL) {
+	 if (IS_LABEL(*bboxgen)) {
 	    labelptr btext = TOLABEL(bboxgen);
 	    if (btext->pin && !(btext->justify & PINVISIBLE)) {
 	       goto nextgen;
@@ -1781,7 +1781,7 @@ void UDrawPath(pathptr thepath)
    
    for (genpath = thepath->plist; genpath < thepath->plist + thepath->parts;
 	  genpath++) {
-      switch((*genpath)->type) {
+      switch(ELEMENTTYPE(*genpath)) {
 	 case POLYGON:
 	    thepoly = TOPOLY(genpath);
 	    pathsegs += thepoly->number;
@@ -1863,6 +1863,8 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
      for (areagen = theobject->plist; areagen < theobject->plist +
            theobject->parts; areagen++) {
 
+       if ((*areagen)->type & DRAW_HIDE) continue;
+
        if (defaultcolor != DOFORALL) {
 	  if ((*areagen)->color != curcolor) {
 	     if ((*areagen)->color == DEFAULTCOLOR)
@@ -1873,7 +1875,7 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 	  }
        }
 
-       switch((*areagen)->type) {
+       switch(ELEMENTTYPE(*areagen)) {
 	  case(POLYGON):
 	     if (level == 0 || !((TOPOLY(areagen))->style & BBOX))
                 UDrawPolygon(TOPOLY(areagen));
@@ -1891,7 +1893,7 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 	     UDrawPath(TOPATH(areagen));
 	     break;
    
-          case(OBJECT):
+          case(OBJINST):
 	     if (areastruct.editinplace && stack && (TOOBJINST(areagen)
 			== areastruct.topinstance)) {
 		/* If stack matches areastruct.stack, then don't draw */
@@ -1960,8 +1962,8 @@ void UDoLatex(objinstptr theinstance, short level, FILE *f,
    for (areagen = theobject->plist; areagen < theobject->plist +
            theobject->parts; areagen++) {
 
-      switch((*areagen)->type) {
-         case(OBJECT):
+      switch(ELEMENTTYPE(*areagen)) {
+         case(OBJINST):
             UDoLatex(TOOBJINST(areagen), level + 1, f, scale, tx, ty, checkonly);
 	    break;
    
