@@ -760,7 +760,7 @@ void composelib(short mode)
 	 (*drawname)->string->data.font = fval;
 	 strptr = makesegment(&((*drawname)->string), NULL);
 	 strptr->type = TEXT_STRING;
-         strptr->data.string = mystrdup(libobj->name);
+         strptr->data.string = strdup(libobj->name);
          (*drawname)->justify = TOP | NOTBOTTOM | NOTLEFT;
 
          if (width > 170)
@@ -1221,7 +1221,7 @@ void copycat()
       for (ops = oldobj->params; ops != NULL; ops = ops->next) {
 	 newops = (oparamptr)malloc(sizeof(oparam));
 	 newops->next = (*newobj)->params;
-	 newops->key = mystrdup(ops->key);
+	 newops->key = strdup(ops->key);
 	 (*newobj)->params = newops;
 	 newops->type = ops->type;
 	 newops->which = ops->which;
@@ -1236,15 +1236,17 @@ void copycat()
 	       newops->parameter.string = stringcopy(ops->parameter.string);
 	       break;
 	    case XC_EXPR:
-	       newops->parameter.expr = mystrdup(ops->parameter.expr);
+	       newops->parameter.expr = strdup(ops->parameter.expr);
 	       break;
 	 }
       }
 
       (*newobj)->schemtype = oldobj->schemtype;
-      (*newobj)->netlist = NULL;
-      (*newobj)->portlist = NULL;
-      (*newobj)->calllist = NULL;
+      (*newobj)->netnames = NULL;
+      (*newobj)->ports = NULL;
+      (*newobj)->calls = NULL;
+      (*newobj)->polygons = NULL;
+      (*newobj)->labels = NULL;
       (*newobj)->valid = False;
       (*newobj)->traversed = False;
       (*newobj)->devname = NULL;
