@@ -35,16 +35,31 @@ extern char *evaluate_expr(oparamptr);
 
 /* from elements.c: */
 
-extern void copyalleparams(genericptr, genericptr);
-extern void copyparams(objinstptr, objinstptr);
-extern void instancedefaults(objinstptr, objectptr, int, int);
-extern void labeldefaults(labelptr, u_char, int, int);
-extern void poly_add_point(polyptr, XPoint *);
+/* element constructor functions */
+extern labelptr new_label(objinstptr, stringpart *, int, int, int);
+extern labelptr new_simple_label(objinstptr, char *, int, int, int);
+extern labelptr new_temporary_label(objectptr, char *, int, int);
+extern polyptr new_polygon(objinstptr, pointlist *, int);
+extern splineptr new_spline(objinstptr, pointlist);
+extern arcptr new_arc(objinstptr, int, int, int);
+extern objinstptr new_objinst(objinstptr, objinstptr, int, int);
+
+/* element destructor function */
+extern void remove_element(objinstptr, genericptr);
+
+/* functions to set default values for element types */
 extern void polydefaults(polyptr, int, int, int);
 extern void splinedefaults(splineptr, int, int);
 extern void arcdefaults(arcptr, int, int);
 extern void pathdefaults(pathptr, int, int);
+extern void instancedefaults(objinstptr, objectptr, int, int);
+extern void labeldefaults(labelptr, u_char, int, int);
+
+extern void poly_add_point(polyptr, XPoint *);
 extern void drawdot(int, int);
+extern void copyalleparams(genericptr, genericptr);
+extern void copyparams(objinstptr, objinstptr);
+
 extern void startspline(xcWidget, caddr_t, caddr_t);
 extern void starttext(xcWidget, pointertype, caddr_t);
 extern void textbutton(u_char, XButtonEvent *);
@@ -52,6 +67,17 @@ extern void charreport(labelptr);
 extern void labeltext(int, char *);
 extern void textreturn();
 extern void rejustify(short);
+extern void finddir(polyptr);
+extern void reversepoints(XPoint *, short);
+extern void reversefpoints(XfPoint *, short);
+extern void freeparts(short *, short);
+extern void removep(short *, short);
+extern void unjoin();
+extern labelptr findlabelcopy(labelptr, stringpart *);
+extern Boolean neartest(XPoint *, XPoint *);
+extern void join();
+
+/* interactive manipulation of elements */
 extern void splinebutton(XButtonEvent *);
 extern void trackspline(xcWidget, caddr_t, caddr_t);
 extern void startarc(xcWidget, caddr_t, caddr_t);
@@ -62,17 +88,8 @@ extern void boxbutton(XButtonEvent *);
 extern void trackbox(xcWidget, caddr_t, caddr_t);
 extern void trackwire(xcWidget, caddr_t, caddr_t);
 extern void startwire(XPoint);
-extern void finddir(polyptr);
 extern void trackpoly(xcWidget, caddr_t, caddr_t);
 extern void setendpoint(short *, short, XPoint **, XPoint *);
-extern void reversepoints(XPoint *, short);
-extern void reversefpoints(XfPoint *, short);
-extern void freeparts(short *, short);
-extern void removep(short *, short);
-extern void unjoin();
-extern labelptr findlabelcopy(labelptr, stringpart *);
-extern Boolean neartest(XPoint *, XPoint *);
-extern void join();
 extern void wirebutton(XButtonEvent *);
 
 /* from events.c: */
@@ -150,6 +167,7 @@ extern short *xc_undelete(objinstptr, objectptr, short);
 extern objectptr delete_element(objinstptr, short *, int, short);
 extern void printname(objectptr);
 extern Boolean checkname(objectptr);
+extern char *checkpostscriptname(char *, int *, objectptr);
 extern objectptr finddot();
 extern void movepoints(genericptr *, short, short);
 extern void domakeobject(xcWidget, caddr_t);
@@ -236,6 +254,7 @@ void importfromlibrary(short, char *, char *);
 objectptr *new_library_object(short, char *, objlistptr *);
 Boolean library_object_unique(short, objectptr, objlistptr);
 void add_object_to_library(short, objectptr);
+u_char *find_delimiter(u_char *);
 
 #ifdef TCL_WRAPPER
 void setfile(char *, int);
@@ -519,9 +538,11 @@ extern Boolean RemoveFromNetlist(objectptr, genericptr);
 extern labelptr NetToLabel(int, objectptr);
 extern void NameToPosition(objinstptr, labelptr, XPoint *);
 extern void invalidate_netlist(objectptr);
+extern void remove_netlist_element(objectptr, genericptr);
 extern int updatenets(objinstptr);
 extern void createnets(objinstptr);
 extern void destroynets();
+extern void gencalls(objectptr);
 extern Boolean nonnetwork(polyptr);
 extern int globalmax();
 extern void gennetlist(objectptr, objinstptr);
@@ -534,24 +555,24 @@ extern int pushnetwork(pushlistptr, objectptr);
 extern int onsegment(XPoint *, XPoint *, XPoint *);
 extern Boolean neardist(long);
 extern Boolean nearpoint(XPoint *, XPoint *);
-extern int make_tmp_pin(objectptr, XPoint *);
-extern Boolean searchconnect(XPoint *, int, objinstptr);
+extern int make_tmp_pin(objectptr, XPoint *, objinstptr, int);
+extern int searchconnect(XPoint *, int, objinstptr, int);
 extern int translateup(int, objectptr, objectptr, objinstptr);
-extern void addpoly(objectptr, polyptr, int);
+extern void addpoly(objectptr, polyptr, int, int);
 extern long zsign(long, long);
-extern void mergecleanup(objectptr);
-extern void mergenets(objectptr, int, int);
+extern void mergenets(objectptr, int, int, int);
 extern void removecall(objectptr, CalllistPtr);
-extern void addpin(objectptr, labelptr, int);
+extern void addpin(objectptr, labelptr, int, int);
 extern void addcall(objectptr, objectptr, objinstptr);
 extern Boolean addport(objectptr, int);
 extern void addportcall(objectptr, objinstptr, objectptr, int, int);
 extern void makelocalpins(objectptr, CalllistPtr, char *);
 extern int porttonet(objectptr, int);
-extern int pointtonet(objectptr, XPoint *);
+extern int pointtonet(objectptr, XPoint *, int, int, objinstptr);
+extern int *pointtolist(objectptr, XPoint *, int *);
 extern stringpart *nettopin(int, objectptr, char *);
-extern int pintonet(objectptr, objinstptr, labelptr);
-extern int nametonet(objectptr, objinstptr, char *);
+extern int pintonet(objectptr, objinstptr, labelptr, int);
+extern linkedNet *nametonet(objectptr, objinstptr, char *);
 extern char *parseinfo(objectptr, CalllistPtr, char *, char *, Boolean);
 extern int writedevice(FILE *, char *, objectptr, CalllistPtr, char *);
 extern void writeflat(objectptr, CalllistPtr, char *, FILE *, char *);
@@ -561,15 +582,17 @@ extern void writenet(objectptr, char *, char *);
 extern void writepcb(struct Ptab **, objectptr, CalllistPtr, char *, char *);
 extern void outputpcb(struct Ptab *, FILE *);
 extern void freepcb(struct Ptab *);
-extern void freepolylist(NetlistPtr);
-extern void freelabellist(NetlistPtr);
+extern void freepolylist(PolylistPtr *);
+extern void freelabellist(LabellistPtr *);
 extern void freeports(CalllistPtr);
 extern void freenets(objectptr);
 extern void freetemplabels(objectptr);
 extern void freeglobals();
-extern int cleartraversed(objectptr);
-extern int checkvalid(objectptr);
-extern void clearpins(objectptr);
+extern void destroynets(objectptr);
+extern int  cleartraversed(objectptr);
+extern int  checkvalid(objectptr);
+extern void clearlocalpins(objectptr);
+extern void freelinkednet(linkedNet **);
 
 /* from parameter.c: */ 
 
@@ -725,6 +748,7 @@ extern stringpart *findstringpart(int, int *, stringpart *, objinstptr);
 extern void charprint(char *, stringpart *, int);
 extern char *stringprint(stringpart *, objinstptr);
 extern char *textprint(stringpart *, objinstptr);
+extern char *textprintsubnet(stringpart *, objinstptr, int);
 extern int textcomp(stringpart *, char *, objinstptr);
 extern int textncomp(stringpart *, char *, objinstptr);
 extern int stringcomp(stringpart *, stringpart *);
@@ -735,6 +759,11 @@ extern stringpart *stringcopy(stringpart *);
 extern stringpart *stringcopyall(stringpart *, objinstptr);
 extern stringpart *stringcopyback(stringpart *, objinstptr);
 extern stringpart *deletestring(stringpart *, stringpart **, objinstptr);
+extern int *break_up_bus(labelptr, objinstptr, int *);
+extern int sub_bus_idx(labelptr, objinstptr);
+extern Boolean pin_is_bus(labelptr, objinstptr);
+extern int find_cardinal(int, labelptr, objinstptr);
+extern int find_ordinal(int, labelptr, objinstptr);
 
 extern short UDrawChar(u_char, short, short, int, int);
 extern void UDrawString(labelptr, int, objinstptr);
