@@ -277,7 +277,7 @@ int connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 
    if (ggen != NULL) {
       if (checkvalid(topobject) == -1) {
-         destroynets();
+         destroynets(topobject);
          createnets(areastruct.topinstance);
       }
       if ((netid = is_resolved(&ggen, seltop, &nettop)) != 0) {
@@ -633,7 +633,7 @@ void collectsubschems(int toppage)
 	  if (xobjs.pagelist[pageno]->filename != NULL)
 	     free(xobjs.pagelist[pageno]->filename);
 	  xobjs.pagelist[pageno]->filename =
-	     mystrdup(xobjs.pagelist[loctop]->filename);
+	     strdup(xobjs.pagelist[loctop]->filename);
       }
    } 
    free((char *)pagelist);
