@@ -5,7 +5,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>   /* for isprint() */
+#include <ctype.h>   /* for isprint() and isdigit() */
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -58,7 +58,7 @@ short del;
 char *evaluate_expr(oparamptr ops)
 {
    if (ops->type != XC_EXPR) return NULL;
-   return mystrdup(ops->parameter.expr);
+   return strdup(ops->parameter.expr);
 }
 
 #endif
@@ -523,6 +523,36 @@ char *textprint(stringpart *strtop, objinstptr localinst)
    return xcstringtostring(strtop, localinst, True);
 }
 
+/*----------------------------------------------------------------------*/
+/* textprintsubnet() is like textprint(), except that strings in bus	*/
+/* notation are reduced to just the single subnet.			*/
+/*----------------------------------------------------------------------*/
+
+char *textprintsubnet(stringpart *strtop, objinstptr localinst, int subnet)
+{
+   char *newstr, *busptr, *endptr, *substr;
+
+   newstr = xcstringtostring(strtop, localinst, True);
+   if (subnet >= 0) {
+      busptr = strchr(newstr, areastruct.buschar);
+      if (busptr != NULL) {
+	 endptr = find_delimiter(busptr);
+	 if (endptr != NULL) {
+	    if (busptr == newstr)
+	       sprintf(newstr, "%d", subnet);
+	    else {
+	       substr = strdup(newstr);
+	       busptr++;
+	       sprintf(substr + (int)(busptr - newstr), "%d%s", subnet, endptr);
+	       free(newstr);
+	       return substr;
+	    }
+	 }
+      }
+   }
+   return newstr;
+}
+
 /*----------------------------------------------------------------------*/
 /* Test equivalence of the text string parts of a label with the	*/
 /* indicated char * string.						*/
@@ -635,18 +665,174 @@ Boolean isisolatin1(int fontnumber)
    return False;
 }
 
+/*----------------------------------------------------------------------*/
+/* For a label representing a single bus subnet, return the index of	*/
+/* the subnet.								*/
+/*----------------------------------------------------------------------*/
+
+int sub_bus_idx(labelptr thislab, objinstptr thisinst)
+{
+   stringpart *strptr;
+   char *busptr;
+   int busidx;
+
+   for (strptr = thislab->string; strptr != NULL; strptr =
+		nextstringpart(strptr, thisinst)) {
+      if (strptr->type == TEXT_STRING) {
+	 if ((busptr = strchr(strptr->data.string, areastruct.buschar)) != NULL) {
+	    if (sscanf(++busptr, "%d", &busidx) == 1)
+	       return busidx;
+	 }
+	 if (sscanf(strptr->data.string, "%d", &busidx) == 1)
+	    return busidx;
+      }
+   }
+   return -1;
+}
+
+/*----------------------------------------------------------------------*/
+/* The following routine is like sub_bus_idx but returns TRUE or FALSE	*/
+/* depending on whether the label was determined to be in bus notation	*/
+/* or not.  Note that sub_bux_idx may be run on sub-bus names (those	*/
+/* that are internally generated from labels in bus notation), but	*/
+/* pin_is_bus should not, because pin numbers in bus notation get the	*/
+/* bus delimiters stripped from them.					*/
+/*----------------------------------------------------------------------*/
+
+Boolean pin_is_bus(labelptr thislab, objinstptr thisinst)
+{
+   stringpart *strptr;
+   char *busptr;
+   int busidx;
+   Boolean found_delimiter = FALSE;
+
+   for (strptr = thislab->string; strptr != NULL; strptr =
+		nextstringpart(strptr, thisinst)) {
+      if (strptr->type == TEXT_STRING) {
+	 if ((busptr = strchr(strptr->data.string, areastruct.buschar)) != NULL) {
+	    if (isdigit(*(++busptr)))
+	       return TRUE;
+	    else
+	       found_delimiter = TRUE;
+	 }
+	 else if (found_delimiter == TRUE) {
+	    return (isdigit(*(strptr->data.string))) ? TRUE : FALSE;
+	 }
+      }
+   }
+   return FALSE;
+}
+
+/*----------------------------------------------------------------------*/
+/* When encountering a label with bus notation, create a list of	*/
+/* subnets belonging to the bus.  Return the list and set numsubnets	*/
+/* to the size of the list (number of subnets).				*/
+/* Unlike the above routine, this routine prints the original string	*/
+/* into a char* array using textprint() so that escape sequences are	*/
+/* removed and will not affect the result.				*/
+/*									*/
+/* To speed things up, the calling routine should have already called	*/
+/* pin_is_bus() to determine if the label does indeed represent a bus.	*/
+/* break_up_bus() is much slower in determining this.  If break_up_bus	*/
+/* is passed a string that cannot be identified as a bus, then it	*/
+/* returns a NULL list.							*/
+/*----------------------------------------------------------------------*/
+
+int *break_up_bus(labelptr blab, objinstptr thisinst, int *numsubnets)
+{
+   int *sublist = NULL;
+   char *tptr;
+   int bpos, istart, iend, tlen, i;
+   char *buspos, *busend, *busptr;
+   int subnetcount = 0;
+
+   if (pin_is_bus(blab, thisinst) == FALSE) return NULL;
+
+   tptr = textprint(blab->string, thisinst);
+   tlen = strlen(tptr) + 1;
+   buspos = strchr(tptr, areastruct.buschar);
+
+   /* The notation "(...)" with NO TEXT preceding the opening bus	*/
+   /* delimiter, is assumed to represent a numerical pin range.  So	*/
+   /* instead of generating labels, e.g., "(1)", "(2)", etc., we	*/
+   /* generate	labels "1", "2", etc.					*/
+
+   if (buspos == NULL) {
+      Fprintf("Error:  Bus specification has no start delimiter!\n");
+      goto doneBus;
+   }
+
+   bpos = (int)(buspos - tptr);
+   busend = find_delimiter(buspos);
+
+   if (busend == NULL) {
+      Fprintf("Error:  Bus specification has no end delimiter!\n");
+      goto doneBus;
+   }
+
+   /* Find the range of each bus */
+
+   istart = -1;
+   for (busptr = buspos + 1; busptr < busend; busptr++) {
+      if (sscanf(busptr, "%d", &iend) == 0) break;
+      while ((*busptr != ':') && (*busptr != '-') && (*busptr != ',')
+		&& (*busptr != *busend))
+	 busptr++;
+      if ((*busptr == ':') || (*busptr == '-')) 	/* numerical range */
+	 istart = iend;
+      else {
+	 if (istart < 0) istart = iend;
+	 i = istart;
+	 while (1) {
+
+	    /* Create a new list entry for this subnet number */
+
+	    subnetcount++;
+	    if (sublist == NULL)
+	       sublist = (int *)malloc(sizeof(int));
+	    else
+	       sublist = (int *)realloc(sublist, subnetcount * sizeof(int));
+
+	    *(sublist + subnetcount - 1) = i;
+
+	    if (i == iend) break;
+	    else if (istart > iend) i--;
+	    else i++;
+	 }
+	 istart = -1;
+      }
+   }
+
+doneBus:
+   free(tptr);
+   *numsubnets = subnetcount;
+   return sublist;
+}
+
 /*----------------------------------------------------------------------*/
 /* Test equivalence of two label strings (relaxed constraints)		*/
 /*    Like stringcomp(), but ignores "superficial" differences such as  */
 /*    color and font (unless one of the fonts is Symbol), scale,	*/
 /*    underlining, tabbing, and kerning.				*/
+/*									*/
+/* Return 0 if matching.  Return 1 if not matching.			*/
+/*									*/
+/* For bus notation, ignore everything inside the bus delimiters.	*/
+/* The calling routine must determine if the labels being compared	*/
+/* have matching subnet ranges.  Note that as written, this does not	*/
+/* check any text occurring after the opening bus delimiter!  Thus,	*/
+/* "mynet(1:2)" and "mynet(3:4)" and "mynet(1)_alt" are all considered	*/
+/* exact matches in bus notation.					*/
 /*----------------------------------------------------------------------*/
 
 int stringcomprelaxed(stringpart *string1, stringpart *string2,
 			objinstptr thisinst)
 {
    stringpart *strptr1 = string1, *strptr2 = string2;
-   Boolean font1 = False, font2 = False;
+   Boolean font1 = False, font2 = False, inbus_match = TRUE;
+   int in_bus = 0;
+   char *buspos;
+   int bpos;
 
    if (strptr1->type == FONT_NAME)
       font1 = issymbolfont(strptr1->data.font);
@@ -672,7 +858,47 @@ int stringcomprelaxed(stringpart *string1, stringpart *string2,
       else {
          switch (strptr1->type) {
 	    case TEXT_STRING:
-	       if (strcmp(strptr1->data.string, strptr2->data.string)) return 1;
+	       if (in_bus == 1) {
+		  char matchchar = areastruct.buschar;
+		  switch (areastruct.buschar) {
+		     case '(': matchchar = ')'; break;
+		     case '[': matchchar = ']'; break;
+		     case '{': matchchar = '}'; break;
+		     case '<': matchchar = '>'; break;
+		  }
+		  buspos = strchr(strptr1->data.string, matchchar);
+		  if (buspos != NULL) {
+		     bpos = (int)(buspos - (char *)(strptr1->data.string));
+		     if (strlen(strptr2->data.string) > bpos) {
+		        if (!strcmp(strptr1->data.string + bpos,
+			  	strptr2->data.string + bpos)) {
+		           in_bus = 2;
+		           break;
+			}
+		     }
+		     return 1;
+		  }
+		  if (inbus_match == TRUE)
+		     if (strcmp(strptr1->data.string, strptr2->data.string))
+			 inbus_match = FALSE;
+	       }
+	       else if (!strcmp(strptr1->data.string, strptr2->data.string))
+		  break;
+
+	       /* To be a matching bus, the strings match everywhere	*/
+	       /* except between the bus notation delimiters (default	*/
+	       /* "()".							*/
+		
+	       buspos = strchr(strptr1->data.string, areastruct.buschar);
+	       if (buspos != NULL) {
+
+		  bpos = (int)(buspos - (char *)(strptr1->data.string)) + 1;
+		  if (!strncmp(strptr1->data.string, strptr2->data.string, bpos)) {
+		     in_bus = 1;
+		     break;
+		  }
+	       }
+	       return 1;  /* strings did not match, exactly or as bus notation */
 	       break;
 	    case OVERLINE:
 	       if (strptr1->type != strptr2->type) return 1;
@@ -685,8 +911,12 @@ int stringcomprelaxed(stringpart *string1, stringpart *string2,
 
    /* One string continues after the other ends. . . */ 
    if (strptr1 != NULL || strptr2 != NULL) return 1;
+
+   /* Treat no closing bus delimiter as a non-bus string */
+   else if ((in_bus == 1) && (inbus_match == FALSE)) return 1;
    return 0;
 }
+
 /*----------------------------------------------------------------------*/
 /* Find the number of parts in a string	(excluding parameter contents)	*/
 /*----------------------------------------------------------------------*/
