@@ -32,9 +32,6 @@ typedef unsigned long	u_long;
   #define calloc(a,b) Tcl_Alloc(a * b)
   #define free(a) Tcl_Free((char *)(a))
   #define realloc(a,b) Tcl_Realloc((char *)(a), b)
-  extern char *mystrdup(const char *);
-#else
-  #define mystrdup(c) strdup(c)
 #endif
 
 /*----------------------------------------------------------------------*/
@@ -63,20 +60,21 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 /*----------------------------------------------------------------------*/
 
 /* Single-bit flags */
-#define	OBJINST	1
-#define LABEL	2
-#define POLYGON	4
-#define ARC	8
-#define SPLINE  16
-#define PATH	32
+#define	OBJINST		1
+#define LABEL		2
+#define POLYGON		4
+#define ARC		8
+#define SPLINE  	16
+#define PATH		32
+
+#define REMOVE_TAG	64	/* element to be removed from netlist	*/
+#define NETLIST_INVALID 128	/* this element invalidates the netlist	*/
+#define SELECT_HIDE	256	/* this element cannot be selected	*/
+#define DRAW_HIDE	512	/* this element is not drawn.		*/
 
-#define REMOVE_TAG 64  /* add this value to objects to be removed */
-#define NETLIST_INVALID 128
-#define SELECT_HIDE     256     /* this element cannot be selected      */
- 
 /* Bit fields */
-#define ALL_TYPES       (OBJINST | LABEL | POLYGON | ARC | SPLINE | PATH)
-#define VALID_TYPES     (REMOVE_TAG | ALL_TYPES)
+#define ALL_TYPES 	(OBJINST | LABEL | POLYGON | ARC | SPLINE | PATH)
+#define VALID_TYPES 	(REMOVE_TAG | ALL_TYPES)
 
 /*----------------------------------------------------------------------*/
 /* Definition shortcuts 						*/
@@ -684,14 +682,36 @@ typedef struct {
 /* Netlist structures for schematic capture				*/
 /*----------------------------------------------------------------------*/
 
+/* Structure to hold net and subnet IDs for a bus */
+
+typedef struct {
+   int netid;
+   int subnetid;
+} buslist;
+
+/* Linked array of nets returned by pintonet() and pointtonet(), to	*/
+/* deal with buses by translating them into multiple networks.		*/
+
+typedef struct _linkedNet *linkedNetPtr;
+typedef struct _linkedNet {
+   int netid;
+   int subnetid;
+   linkedNetPtr next;
+} linkedNet;
+
 /* Linked polygon list */
 
 typedef struct _Polylist *PolylistPtr;
 typedef struct _Polylist
 {
-   objectptr cschem;	/* Schematic containing the polygon */
+   objectptr cschem;	/* Schematic containing the polygon	*/
+   union {
+      int  id;		/* A single net ID, if subnets == 0	*/
+      buslist *list;	/* List of net and subnet IDs for a bus	*/
+   } net;
+   int subnets;		/* Number of subnets; 0 if no subnets	*/
    polyptr poly;
-   PolylistPtr next;	/* Next polygon in the linked list */
+   PolylistPtr next;	/* Next polygon in the linked list	*/
 } Polylist;
 
 /* Linked label list */
@@ -699,22 +719,25 @@ typedef struct _Polylist
 typedef struct _Labellist *LabellistPtr;
 typedef struct _Labellist
 {
-   objectptr cschem;	/* Schematic containing the label */
+   objectptr cschem;	/* Schematic containing the label	*/
+   union {
+      int  id;		/* A single net ID, if subnets == 0	*/
+      buslist *list;	/* List of net and subnet IDs for a bus	*/
+   } net;
+   int subnets;		/* Number of subnets; 0 if no subnets	*/
    labelptr label;
    LabellistPtr next;
 } Labellist;
 
-/* List of object's networks */
+/* List of object's networks by (flattened) name */
 
-typedef struct _Netlist *NetlistPtr;
-typedef struct _Netlist
+typedef struct _Netname *NetnamePtr;
+typedef struct _Netname
 {
    int netid;
-   PolylistPtr polygons;
-   LabellistPtr labels;
    stringpart *localpin;
-   NetlistPtr next;
-} Netlist;
+   NetnamePtr next;
+} Netname;
 
 /* List of object's I/O ports */
 
@@ -791,9 +814,12 @@ typedef struct _xcobject {
    char		*devname;	/* if non-null, name of device in netlist */
    Boolean	valid;		/* Is current netlist valid? */
    Boolean	traversed;	/* Flag to indicate object was processed */
-   NetlistPtr   netlist;
-   PortlistPtr  portlist;
-   CalllistPtr  calllist;
+   LabellistPtr	labels;		/* Netlist pins	 */
+   PolylistPtr	polygons;	/* Netlist wires */
+   PortlistPtr  ports;		/* Netlist ports */
+   CalllistPtr  calls;		/* Netlist subcircuits and connections */
+   NetnamePtr   netnames;	/* Local names for flattening */
+				/* (this probably shouldn't be here. . .) */
 } object;
 
 /*----------------------------------------------------------------------*/
@@ -1117,6 +1143,7 @@ typedef struct {
    Boolean	invert;
    Boolean	mapped;		/* indicates if window is drawable */
    Boolean	filefilter;	/* Is the file list filtered? */
+   char		buschar;	/* Character indicating vector notation */
    Boolean	editinplace;
    Boolean	schemon;
    Boolean	pinpointon;
